@@ -10,7 +10,7 @@
    This file is part of LibVEX, a library for dynamic binary
    instrumentation and translation.
 
-   Copyright (C) 2004-2005 OpenWorks LLP.  All rights reserved.
+   Copyright (C) 2004-2007 OpenWorks LLP.  All rights reserved.
 
    This library is made available under a dual licensing scheme.
 
@@ -52,6 +52,7 @@
 #include "../pub/libvex_guest_x86.h"
 #include "../pub/libvex_guest_amd64.h"
 #include "../pub/libvex_guest_ppc32.h"
+#include "../pub/libvex_guest_ppc64.h"
 
 Int main ( void )
 {
@@ -168,5 +169,35 @@ Int main ( void )
 
   printf("\n");
 
+  // ppc64
+  printf("#define OFFSET_ppc64_GPR0     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR0));
+
+  printf("#define OFFSET_ppc64_GPR3     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR3));
+
+  printf("#define OFFSET_ppc64_GPR4     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR4));
+
+  printf("#define OFFSET_ppc64_GPR5     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR5));
+
+  printf("#define OFFSET_ppc64_GPR6     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR6));
+
+  printf("#define OFFSET_ppc64_GPR7     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR7));
+
+  printf("#define OFFSET_ppc64_GPR8     %4d\n",
+         offsetof(VexGuestPPC64State,guest_GPR8));
+
+  printf("#define OFFSET_ppc64_CIA      %4d\n",
+         offsetof(VexGuestPPC64State,guest_CIA));
+
+  printf("#define OFFSET_ppc64_CR0_0    %4d\n",
+         offsetof(VexGuestPPC64State,guest_CR0_0));
+
+  printf("\n");
+
   return 0;
 }
