@@ -7,9 +7,9 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2006 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
-   Copyright (C) 2003-2006 Jeremy Fitzhardinge
+   Copyright (C) 2003-2007 Jeremy Fitzhardinge
       jeremy@goop.org
 
    This program is free software; you can redistribute it and/or
@@ -762,6 +762,10 @@ void VG_(redir_initialise) ( void )
          "ld.so.1", "strcmp",
          (Addr)&VG_(ppc32_linux_REDIR_FOR_strcmp)
       );
+      add_hardwired_spec(
+         "ld.so.1", "index",
+         (Addr)&VG_(ppc32_linux_REDIR_FOR_strchr)
+      );
    }
 
 #  elif defined(VGP_ppc64_linux)
