@@ -8,7 +8,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -71,8 +71,14 @@ struct elfinfo
 static void check_mmap(SysRes res, Addr base, SizeT len)
 {
    if (res.isError) {
-      VG_(printf)("valgrind: mmap(0x%llx, %lld) failed in UME.\n", 
-                  (ULong)base, (Long)len);
+      VG_(printf)("valgrind: mmap(0x%llx, %lld) failed in UME "
+                  "with error %d (%s).\n", 
+                  (ULong)base, (Long)len, 
+                  res.val, VG_(strerror)(res.val) );
+      if (res.val == VKI_EINVAL) {
+         VG_(printf)("valgrind: this can be caused by executables with "
+                     "very large text, data or bss segments.\n");
+      }
       VG_(exit)(1);
    }
 }
@@ -93,7 +99,7 @@ struct ume_auxv *VG_(find_auxv)(UWord* sp)
       sp++;
    sp++;
    
-#if defined(VGA_ppc32)
+#if defined(VGA_ppc32) || defined(VGA_ppc64)
 # if defined AT_IGNOREPPC
    while (*sp == AT_IGNOREPPC)        // skip AT_IGNOREPPC entries
       sp += 2;
@@ -315,7 +321,7 @@ static Bool match_ELF(const char *hdr, Int len)
 
    - The entry point in INFO is set to the interpreter's entry point,
      and we're done.  */
-static Int load_ELF(Int fd, const char *name, /*MOD*/struct exeinfo *info)
+static Int load_ELF(Int fd, const HChar* name, /*MOD*/ExeInfo* info)
 {
    SysRes sres;
    struct elfinfo *e;
@@ -329,6 +335,11 @@ static Int load_ELF(Int fd, const char *name, /*MOD*/struct exeinfo *info)
    void *entry;
    ESZ(Addr) ebase = 0;
 
+   /* The difference between where the interpreter got mapped and
+      where it asked to be mapped.  Needed for computing the ppc64 ELF
+      entry point and initial tocptr (R2) value. */
+   ESZ(Word) interp_offset = 0;
+
 #ifdef HAVE_PIE
    ebase = info->exe_base;
 #endif
@@ -484,6 +495,7 @@ static Int load_ELF(Int fd, const char *name, /*MOD*/struct exeinfo *info)
 
       entry = (void *)(advised - interp_addr + interp->e.e_entry);
       info->interp_base = (ESZ(Addr))advised;
+      interp_offset = advised - interp_addr;
 
       VG_(free)(interp->p);
       VG_(free)(interp);
@@ -493,8 +505,19 @@ static Int load_ELF(Int fd, const char *name, /*MOD*/struct exeinfo *info)
    info->exe_base = minaddr + ebase;
    info->exe_end  = maxaddr + ebase;
 
-   info->init_eip = (Addr)entry;
-
+#if defined(VGP_ppc64_linux)
+   /* On PPC64, a func ptr is represented by a TOC entry ptr.  This
+      TOC entry contains three words; the first word is the function
+      address, the second word is the TOC ptr (r2), and the third word
+      is the static chain value. */
+   info->init_ip  = ((ULong*)entry)[0];
+   info->init_toc = ((ULong*)entry)[1];
+   info->init_ip  += interp_offset;
+   info->init_toc += interp_offset;
+#else
+   info->init_ip  = (Addr)entry;
+   info->init_toc = 0; /* meaningless on this platform */
+#endif
    VG_(free)(e->p);
    VG_(free)(e);
 
@@ -525,12 +548,12 @@ static Bool match_script(char *hdr, Int len)
 }
 
 // Forward declaration.
-static Int do_exec_inner(const char *exe, struct exeinfo *info);
+static Int do_exec_inner(const HChar* exe, ExeInfo* info);
 
 /* returns: 0 = success, non-0 is failure */
-static Int load_script(Int fd, const char *name, struct exeinfo *info)
+static Int load_script(Int fd, const HChar* name, ExeInfo* info)
 {
-   Char  hdr[VKI_PAGE_SIZE];
+   Char  hdr[VKI_MAX_PAGE_SIZE];
    Int   len = VKI_PAGE_SIZE;
    Int   eol;
    Char* interp;
@@ -601,11 +624,11 @@ typedef enum {
 } ExeFormat;
 
 // Check the file looks executable.
-SysRes VG_(pre_exec_check)(const Char* exe_name, Int* out_fd)
+SysRes VG_(pre_exec_check)(const HChar* exe_name, Int* out_fd)
 {
    Int fd, ret;
    SysRes res;
-   Char  buf[VKI_PAGE_SIZE];
+   Char  buf[VKI_MAX_PAGE_SIZE];
    SizeT bufsz = VKI_PAGE_SIZE, fsz;
 
    // Check it's readable
@@ -656,7 +679,7 @@ SysRes VG_(pre_exec_check)(const Char* exe_name, Int* out_fd)
 // We can execute only ELF binaries or scripts that begin with "#!".  (Not,
 // for example, scripts that don't begin with "#!";  see the VG_(do_exec)()
 // invocation from m_main.c for how that's handled.)
-static Int do_exec_inner(const char *exe, struct exeinfo *info)
+static Int do_exec_inner(const HChar *exe, ExeInfo* info)
 {
    SysRes res;
    Int fd;
@@ -721,8 +744,8 @@ static Bool is_binary_file(Char* f)
 // bash as a guide).  It's worth noting that the shell can execute some
 // things that VG_(do_exec)() (which subsitutes for the kernel's exec())
 // will refuse to (eg. scripts lacking a "#!" prefix).
-static Int do_exec_shell_followup(Int ret, Char* exe_name,
-                                  struct exeinfo* info)
+static Int do_exec_shell_followup(Int ret, HChar* exe_name,
+                                  ExeInfo* info)
 {
    Char*  default_interp_name = "/bin/sh";
    SysRes res;
@@ -775,7 +798,7 @@ static Int do_exec_shell_followup(Int ret, Char* exe_name,
 
       // Otherwise it was something else.
       } else {
-         VG_(printf)("valgrind: %s\n", exe_name, VG_(strerror)(ret));
+         VG_(printf)("valgrind: %s: %s\n", exe_name, VG_(strerror)(ret));
       }
       // 126 means NOEXEC;  I think this is Posix, and that in some cases we
       // should be returning 127, meaning NOTFOUND.  Oh well.
@@ -790,7 +813,7 @@ static Int do_exec_shell_followup(Int ret, Char* exe_name,
 // See ume.h for an indication of which entries of 'info' are inputs, which
 // are outputs, and which are both.
 /* returns: 0 = success, non-0 is failure */
-Int VG_(do_exec)(const char *exe_name, struct exeinfo *info)
+Int VG_(do_exec)(const HChar* exe_name, ExeInfo* info)
 {
    Int ret;
    
