@@ -8,7 +8,7 @@
    This file is part of Helgrind, a Valgrind tool for detecting
    data races in threaded programs.
 
-   Copyright (C) 2002-2005 Nicholas Nethercote
+   Copyright (C) 2002-2007 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -29,6 +29,9 @@
    The GNU General Public License is contained in the file COPYING.
 */
 
+// *** WARNING *** 
+// Helgrind is not 64-bit clean.
+
 // For anyone wanting to understand race conditions better, this paper might
 // be instructive:
 //
@@ -51,7 +54,6 @@
 #include "pub_tool_machine.h"
 #include "pub_tool_mallocfree.h"
 #include "pub_tool_options.h"
-#include "pub_tool_profile.h"
 #include "pub_tool_replacemalloc.h"
 #include "pub_tool_tooliface.h"
 
@@ -1630,8 +1632,6 @@ void set_address_range_state ( Addr a, SizeT len /* in bytes */,
                    "Warning: set address range state: large range %d",
                    len);
 
-   //VGP_PUSHCC(VgpSARP);
-
    /* Remove mutexes in recycled memory range from hash */
    find_mutex_range(a, a+len, cleanmx);
 
@@ -1677,8 +1677,6 @@ void set_address_range_state ( Addr a, SizeT len /* in bytes */,
       VG_(printf)("init_status = %u\n", status);
       VG_(tool_panic)("Unexpected Vge_InitStatus");
    }
-      
-   //VGP_POPCC(VgpSARP);
 }
 
 
@@ -2298,8 +2296,10 @@ UCodeBlock* TL_(instrument) ( UCodeBlock* cb_in, Addr not_used )
 }
 #endif
 static
-IRBB* hg_instrument ( IRBB* bb_in, VexGuestLayout* layout, 
-                      Addr64 orig_addr_noredir, VexGuestExtents* vge,
+IRBB* hg_instrument ( VgCallbackClosure* closure,
+                      IRBB* bb,
+                      VexGuestLayout* layout, 
+                      VexGuestExtents* vge,
                       IRType gWordTy, IRType hWordTy )
 {
    tl_assert(0);  // Need to convert to Vex
@@ -2524,7 +2524,9 @@ static void record_race_error ( ThreadId tid, Addr a, Bool is_write,
    err_extra.prevstate = prevstate;
    if (clo_execontext)
       err_extra.lasttouched = getExeContext(a);
-   err_extra.addrinfo.expr = VG_(describe_addr)(tid, a);
+   /* JRS 4 Apr 06: VG_(describe_addr) disappeared from m_debuginfo,
+      at least for the time being. */
+   err_extra.addrinfo.expr = "???"; /* VG_(describe_addr)(tid, a); */
 
    VG_(maybe_record_error)( tid, RaceErr, a, 
                             (is_write ? "writing" : "reading"),
@@ -3380,7 +3382,7 @@ static void hg_pre_clo_init(void)
    VG_(details_version)         (NULL);
    VG_(details_description)     ("a data race detector");
    VG_(details_copyright_author)(
-      "Copyright (C) 2002-2005, and GNU GPL'd, by Nicholas Nethercote et al.");
+      "Copyright (C) 2002-2007, and GNU GPL'd, by Nicholas Nethercote et al.");
    VG_(details_bug_reports_to)  (VG_BUGS_TO);
    VG_(details_avg_translation_sizeB) ( 115 );
 
@@ -3395,6 +3397,8 @@ static void hg_pre_clo_init(void)
 "     platforms introduced in Valgrind 3.0.0\n"
 " (b) we need to get thread operation tracking working again after\n"
 "     the changes added in Valgrind 2.4.0\n"
+" If you want to use Helgrind, you'll have to use Valgrind 2.2.0, which is\n"
+" the most recent Valgrind release that contains a working Helgrind.\n"
 "\n"
 "Sorry for the inconvenience.  Let us know if this is a problem for you.\n");
    VG_(exit)(1);
