@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -121,14 +121,14 @@ SysRes;
 
 #if defined(VGA_x86) || defined(VGA_amd64)
 #  define VG_LITTLEENDIAN 1
-#elif defined(VGA_ppc32)
+#elif defined(VGA_ppc32) || defined(VGA_ppc64)
 #  define VG_BIGENDIAN 1
 #endif
 
 /* Regparmness */
 #if defined(VGA_x86)
 #  define VG_REGPARM(n)            __attribute__((regparm(n)))
-#elif defined(VGA_amd64) || defined(VGA_ppc32)
+#elif defined(VGA_amd64) || defined(VGA_ppc32) || defined(VGA_ppc64)
 #  define VG_REGPARM(n)            /* */
 #else
 #  error Unknown arch
@@ -138,6 +138,9 @@ SysRes;
 #define VG_STRINGIFZ(__str)  #__str
 #define VG_STRINGIFY(__str)  VG_STRINGIFZ(__str)
 
+// Where to send bug reports to.
+#define VG_BUGS_TO "www.valgrind.org"
+
 #endif /* __PUB_TOOL_BASICS_H */
 
 /*--------------------------------------------------------------------*/
