@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward
+   Copyright (C) 2000-2007 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -59,14 +59,20 @@ extern VgHashTable VG_(HT_construct) ( UInt n_chains );
 extern Int VG_(HT_count_nodes) ( VgHashTable table );
 
 /* Add a node to the table. */
-extern void VG_(HT_add_node) ( VgHashTable t, VgHashNode* node );
+extern void VG_(HT_add_node) ( VgHashTable t, void* node );
 
 /* Looks up a node in the hash table.  Also returns the address of the
    previous node's `next' pointer which allows it to be removed from the
    list later without having to look it up again.  */
-extern VgHashNode* VG_(HT_get_node) ( VgHashTable t, UWord key,
+extern void* VG_(HT_get_node) ( VgHashTable t, UWord key,
                                     /*OUT*/VgHashNode*** next_ptr );
 
+/* Looks up a VgHashNode in the table.  Returns NULL if not found. */
+extern void* VG_(HT_lookup) ( VgHashTable table, UWord key );
+
+/* Removes a VgHashNode from the table.  Returns NULL if not found. */
+extern void* VG_(HT_remove) ( VgHashTable table, UWord key );
+
 /* Allocates an array of pointers to all the shadow chunks of malloc'd
    blocks.  Must be freed with VG_(free)(). */
 extern VgHashNode** VG_(HT_to_array) ( VgHashTable t, /*OUT*/ UInt* n_shadows );
@@ -74,9 +80,9 @@ extern VgHashNode** VG_(HT_to_array) ( VgHashTable t, /*OUT*/ UInt* n_shadows );
 /* Returns first node that matches predicate `p', or NULL if none do.
    Extra arguments can be implicitly passed to `p' using `d' which is an
    opaque pointer passed to `p' each time it is called. */
-extern VgHashNode* VG_(HT_first_match) ( VgHashTable t,
-                                         Bool (*p)(VgHashNode*, void*),
-                                         void* d );
+extern void* VG_(HT_first_match) ( VgHashTable t,
+                                   Bool (*p)(VgHashNode*, void*),
+                                   void* d );
 
 /* Applies a function f() once to each node.  Again, `d' can be used
    to pass extra information to the function. */
@@ -84,6 +90,14 @@ extern void VG_(HT_apply_to_all_nodes)( VgHashTable t,
                                         void (*f)(VgHashNode*, void*),
                                         void* d );
 
+/* Reset the table's iterator to point to the first element. */
+extern void VG_(HT_ResetIter) ( VgHashTable table );
+
+/* Return the element pointed to by the iterator and move on to the next
+   one.  Returns NULL if the last one has been passed, or if HT_ResetIter()
+   has not been called previously. */
+extern void* VG_(HT_Next) ( VgHashTable table );
+
 /* Destroy a table. */
 extern void VG_(HT_destruct) ( VgHashTable t );
 
