@@ -7,7 +7,7 @@
    This file is part of Massif, a Valgrind tool for profiling memory
    usage of programs.
 
-   Copyright (C) 2003-2005 Nicholas Nethercote
+   Copyright (C) 2003-2007 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -35,21 +35,21 @@
 // structures below for more info on how things work.
 
 #include "pub_tool_basics.h"
+#include "pub_tool_aspacemgr.h"
 #include "pub_tool_debuginfo.h"
 #include "pub_tool_hashtable.h"
 #include "pub_tool_libcbase.h"
 #include "pub_tool_libcassert.h"
 #include "pub_tool_libcfile.h"
-#include "pub_tool_libcmman.h"
 #include "pub_tool_libcprint.h"
 #include "pub_tool_libcproc.h"
 #include "pub_tool_machine.h"
 #include "pub_tool_mallocfree.h"
 #include "pub_tool_options.h"
-#include "pub_tool_profile.h"
 #include "pub_tool_replacemalloc.h"
 #include "pub_tool_stacktrace.h"
 #include "pub_tool_tooliface.h"
+#include "pub_tool_clientstate.h"
 
 #include "valgrind.h"           // For {MALLOC,FREE}LIKE_BLOCK
 
@@ -197,25 +197,6 @@ typedef
    }
    HP_Chunk;
 
-/*------------------------------------------------------------*/
-/*--- Profiling events                                     ---*/
-/*------------------------------------------------------------*/
-
-typedef 
-   enum {
-      VgpGetXPt = VgpFini+1,
-      VgpGetXPtSearch,
-      VgpCensus,
-      VgpCensusHeap,
-      VgpCensusSnapshot,
-      VgpCensusTreeSize,
-      VgpUpdateXCon,
-      VgpCalcSpacetime2,
-      VgpPrintHp,
-      VgpPrintXPts,
-   }
-   VgpToolCC;
-
 /*------------------------------------------------------------*/
 /*--- Statistics                                           ---*/
 /*------------------------------------------------------------*/
@@ -367,7 +348,10 @@ static void* perm_malloc(SizeT n_bytes)
    #define SUPERBLOCK_SIZE  (1 << 20)         // 1 MB
 
    if (hp + n_bytes > hp_lim) {
-      hp     = (Addr)VG_(get_memory_from_mmap)(SUPERBLOCK_SIZE, "perm_malloc");
+      hp = (Addr)VG_(am_shadow_alloc)(SUPERBLOCK_SIZE);
+      if (hp == 0)
+         VG_(out_of_memory_NORETURN)( "massif:perm_malloc", 
+                                      SUPERBLOCK_SIZE);
       hp_lim = hp + SUPERBLOCK_SIZE - 1;
    }
 
@@ -438,8 +422,6 @@ static XPt* get_XCon( ThreadId tid, Bool custom_malloc )
    UInt overestimate;
    Bool reached_bottom;
 
-   VGP_PUSHCC(VgpGetXPt);
-
    // Want at least clo_depth non-alloc-fn entries in the snapshot.
    // However, because we have 1 or more (an unknown number, at this point)
    // alloc-fns ignored, we overestimate the size needed for the stack
@@ -486,7 +468,6 @@ static XPt* get_XCon( ThreadId tid, Bool custom_malloc )
       // XXX: linear search, ugh -- about 10% of time for konqueror startup
       // XXX: tried cacheing last result, only hit about 4% for konqueror
       // Nb:  this search hits about 98% of the time for konqueror
-      VGP_PUSHCC(VgpGetXPtSearch);
 
       // If we've searched/added deep enough, or run out of EIPs, this is
       // the bottom XPt.
@@ -514,12 +495,10 @@ static XPt* get_XCon( ThreadId tid, Bool custom_malloc )
          if (ips[L] == xpt->children[nC]->ip) break;   // found the IP
          nC++;                                           // keep looking
       }
-      VGP_POPCC(VgpGetXPtSearch);
 
       // Return found/built bottom-XPt.
       if (reached_bottom) {
          tl_assert(0 == xpt->children[nC]->n_children);   // Must be bottom-XPt
-         VGP_POPCC(VgpGetXPt);
          return xpt->children[nC];
       }
 
@@ -532,8 +511,6 @@ static XPt* get_XCon( ThreadId tid, Bool custom_malloc )
 // Update 'curr_space' of every XPt in the XCon, by percolating upwards.
 static void update_XCon(XPt* xpt, Int space_delta)
 {
-   VGP_PUSHCC(VgpUpdateXCon);
-
    tl_assert(True == clo_heap);
    tl_assert(0    != space_delta);
    tl_assert(NULL != xpt);
@@ -546,8 +523,6 @@ static void update_XCon(XPt* xpt, Int space_delta)
    } 
    if (space_delta < 0) tl_assert(alloc_xpt->curr_space >= -space_delta);
    alloc_xpt->curr_space += space_delta;
-
-   VGP_POPCC(VgpUpdateXCon);
 }
 
 // Actually want a reverse sort, biggest to smallest
@@ -653,28 +628,6 @@ static void* dequeue(Queue* q)
 /*--- malloc() et al replacement wrappers                  ---*/
 /*------------------------------------------------------------*/
 
-static __inline__ 
-void add_HP_Chunk(HP_Chunk* hc)
-{
-   n_heap_blocks++;
-   VG_(HT_add_node) ( malloc_list, (VgHashNode*)hc );
-}
-
-static __inline__ 
-HP_Chunk* get_HP_Chunk(void* p, HP_Chunk*** prev_chunks_next_ptr)
-{
-   return (HP_Chunk*)VG_(HT_get_node) ( malloc_list, (UWord)p,
-                                        (VgHashNode***)prev_chunks_next_ptr );
-}
-
-static __inline__
-void remove_HP_Chunk(HP_Chunk* hc, HP_Chunk** prev_chunks_next_ptr)
-{
-   tl_assert(n_heap_blocks > 0);
-   n_heap_blocks--;
-   *prev_chunks_next_ptr = hc->next;
-}
-
 // Forward declaration
 static void hp_census(void);
 
@@ -686,8 +639,6 @@ void* new_block ( ThreadId tid, void* p, SizeT size, SizeT align,
    Bool custom_alloc = (NULL == p);
    if (size < 0) return NULL;
 
-   VGP_PUSHCC(VgpCliMalloc);
-
    // Update statistics
    n_allocs++;
    if (0 == size) n_zero_allocs++;
@@ -696,13 +647,12 @@ void* new_block ( ThreadId tid, void* p, SizeT size, SizeT align,
    if (!p) {
       p = VG_(cli_malloc)( align, size );
       if (!p) {
-         VGP_POPCC(VgpCliMalloc);
          return NULL;
       }
       if (is_zeroed) VG_(memset)(p, 0, size);
    }
 
-   // Make new HP_Chunk node, add to malloclist
+   // Make new HP_Chunk node, add to malloc_list
    hc       = VG_(malloc)(sizeof(HP_Chunk));
    hc->size = size;
    hc->data = (Addr)p;
@@ -712,31 +662,29 @@ void* new_block ( ThreadId tid, void* p, SizeT size, SizeT align,
       if (0 != size) 
          update_XCon(hc->where, size);
    }
-   add_HP_Chunk( hc );
+   VG_(HT_add_node)(malloc_list, hc);
+   n_heap_blocks++;
 
    // do a census!
    hp_census();      
 
-   VGP_POPCC(VgpCliMalloc);
    return p;
 }
 
 static __inline__
 void die_block ( void* p, Bool custom_free )
 {
-   HP_Chunk *hc, **remove_handle;
+   HP_Chunk* hc;
    
-   VGP_PUSHCC(VgpCliMalloc);
-
    // Update statistics
    n_frees++;
 
-   // Remove HP_Chunk from malloclist
-   hc = get_HP_Chunk( p, &remove_handle );
-   if (hc == NULL)
-      return;   // must have been a bogus free(), or p==NULL
-   tl_assert(hc->data == (Addr)p);
-   remove_HP_Chunk(hc, remove_handle);
+   // Remove HP_Chunk from malloc_list
+   hc = VG_(HT_remove)(malloc_list, (UWord)p);
+   if (NULL == hc)
+      return;   // must have been a bogus free()
+   tl_assert(n_heap_blocks > 0);
+   n_heap_blocks--;
 
    if (clo_heap && hc->size != 0)
       update_XCon(hc->where, -hc->size);
@@ -749,8 +697,6 @@ void die_block ( void* p, Bool custom_free )
 
    // do a census!
    hp_census();
-
-   VGP_POPCC(VgpCliMalloc);
 }
  
 
@@ -796,23 +742,17 @@ static void ms___builtin_vec_delete ( ThreadId tid, void* p )
 
 static void* ms_realloc ( ThreadId tid, void* p_old, SizeT new_size )
 {
-   HP_Chunk*    hc;
-   HP_Chunk**   remove_handle;
-   Int          i;
-   void*        p_new;
-   SizeT        old_size;
-   XPt         *old_where, *new_where;
+   HP_Chunk* hc;
+   void*     p_new;
+   SizeT     old_size;
+   XPt      *old_where, *new_where;
    
-   VGP_PUSHCC(VgpCliMalloc);
-
-   // First try and find the block.
-   hc = get_HP_Chunk ( p_old, &remove_handle );
+   // Remove the old block
+   hc = VG_(HT_remove)(malloc_list, (UWord)p_old);
    if (hc == NULL) {
-      VGP_POPCC(VgpCliMalloc);
-      return NULL;   // must have been a bogus free()
+      return NULL;   // must have been a bogus realloc()
    }
 
-   tl_assert(hc->data == (Addr)p_old);
    old_size = hc->size;
   
    if (new_size <= old_size) {
@@ -822,35 +762,35 @@ static void* ms_realloc ( ThreadId tid, void* p_old, SizeT new_size )
    } else {
       // new size is bigger;  make new block, copy shared contents, free old
       p_new = VG_(cli_malloc)(VG_(clo_alignment), new_size);
-
-      for (i = 0; i < old_size; i++)
-         ((UChar*)p_new)[i] = ((UChar*)p_old)[i];
-
-      VG_(cli_free)(p_old);
-   }
-   
-   old_where = hc->where;
-   new_where = get_XCon( tid, /*custom_malloc*/False);
-
-   // Update HP_Chunk
-   hc->data  = (Addr)p_new;
-   hc->size  = new_size;
-   hc->where = new_where;
-
-   // Update XPt curr_space fields
-   if (clo_heap) {
-      if (0 != old_size) update_XCon(old_where, -old_size);
-      if (0 != new_size) update_XCon(new_where,  new_size);
+      if (p_new) {
+         VG_(memcpy)(p_new, p_old, old_size);
+         VG_(cli_free)(p_old);
+      }
    }
 
-   // If block has moved, have to remove and reinsert in the malloclist
-   // (since the updated 'data' field is the hash lookup key).
-   if (p_new != p_old) {
-      remove_HP_Chunk(hc, remove_handle);
-      add_HP_Chunk(hc);
+   if (p_new) {
+      old_where = hc->where;
+      new_where = get_XCon( tid, /*custom_malloc*/False);
+
+      // Update HP_Chunk
+      hc->data  = (Addr)p_new;
+      hc->size  = new_size;
+      hc->where = new_where;
+
+      // Update XPt curr_space fields
+      if (clo_heap) {
+         if (0 != old_size) update_XCon(old_where, -old_size);
+         if (0 != new_size) update_XCon(new_where,  new_size);
+      }
    }
 
-   VGP_POPCC(VgpCliMalloc);
+   // Now insert the new hc (with a possibly new 'data' field) into
+   // malloc_list.  If this realloc() did not increase the memory size, we
+   // will have removed and then re-added mc unnecessarily.  But that's ok
+   // because shrinking a block with realloc() is (presumably) much rarer
+   // than growing it, and this way simplifies the growing case.
+   VG_(HT_add_node)(malloc_list, hc);
+
    return p_new;
 }
 
@@ -862,13 +802,6 @@ static void* ms_realloc ( ThreadId tid, void* p_old, SizeT new_size )
 static Census censi[MAX_N_CENSI];
 static UInt   curr_census = 0;
 
-// Must return False so that all stacks are traversed
-static Bool count_stack_size( Addr stack_min, Addr stack_max, void *cp )
-{
-   *(UInt *)cp  += (stack_max - stack_min);
-   return False;
-}
-
 static UInt get_xtree_size(XPt* xpt, UInt ix)
 {
    UInt i;
@@ -1000,14 +933,11 @@ static void hp_census(void)
    Int     ms_time, ms_time_since_prev;
    Census* census;
 
-   VGP_PUSHCC(VgpCensus);
-
    // Only do a census if it's time
    ms_time            = VG_(read_millisecond_timer)();
    ms_time_since_prev = ms_time - ms_prev_census;
    if (ms_time < ms_next_census) {
       n_fake_censi++;
-      VGP_POPCC(VgpCensus);
       return;
    }
    n_real_censi++;
@@ -1033,8 +963,6 @@ static void hp_census(void)
       VG_(ssort)(alloc_xpt->children, alloc_xpt->n_children, sizeof(XPt*),
                  XPt_cmp_approx_ST);
 
-      VGP_PUSHCC(VgpCensusHeap);
-
       // For each significant top-level XPt, record space info about its
       // entire XTree, in a single census entry.
       // Nb: the xtree_size count/snapshot buffer allocation, and the actual
@@ -1044,9 +972,7 @@ static void hp_census(void)
          UInt xtree_size, xtree_size2;
 //         VG_(printf)("%7u ", alloc_xpt->children[i]->approx_ST);
          // Count how many XPts are in the XTree
-         VGP_PUSHCC(VgpCensusTreeSize);
          xtree_size = get_xtree_size( alloc_xpt->children[i], 0 );
-         VGP_POPCC(VgpCensusTreeSize);
 
          // If no XPts counted (ie. alloc_xpt.curr_space==0 or XTree
          // insignificant) then don't take any more snapshots.
@@ -1065,18 +991,14 @@ static void hp_census(void)
          // (Except for ones with curr_space==0, which wouldn't contribute
          // to the final exact_ST_dbld calculation anyway;  excluding them
          // saves a lot of memory and up to 40% time with big --depth valus.
-         VGP_PUSHCC(VgpCensusSnapshot);
          xtree_size2 = do_space_snapshot(alloc_xpt->children[i],
                                          census->xtree_snapshots[i], 0);
          tl_assert(xtree_size == xtree_size2);
-         VGP_POPCC(VgpCensusSnapshot);
       }
 //      VG_(printf)("\n\n");
       // Zero-terminate 'xtree_snapshot' array
       census->xtree_snapshots[i] = NULL;
 
-      VGP_POPCC(VgpCensusHeap);
-
       //VG_(printf)("printed %d censi\n", K);
 
       // Lump the rest into a single "others" entry.
@@ -1092,9 +1014,13 @@ static void hp_census(void)
 
    // Stack(s) ---------------------------------------------------------
    if (clo_stacks) {
+      ThreadId tid;
+      Addr     stack_min, stack_max;
       census->stacks_space = sigstacks_space;
-      // slightly abusing this function
-      VG_(first_matching_thread_stack)( count_stack_size, &census->stacks_space );
+      VG_(thread_stack_reset_iter)();
+      while ( VG_(thread_stack_next)(&tid, &stack_min, &stack_max) ) {
+         census->stacks_space += (stack_max - stack_min);
+      }
    }
 
    // Finish, update interval if necessary -----------------------------
@@ -1119,8 +1045,6 @@ static void hp_census(void)
    //ms_next_census += ms_interval;
 
    //VG_(printf)("Next: %d ms\n", ms_next_census);
-
-   VGP_POPCC(VgpCensus);
 } 
 
 /*------------------------------------------------------------*/
@@ -1170,8 +1094,12 @@ static Bool ms_handle_client_request ( ThreadId tid, UWord* argv, UWord* ret )
 /*--- Instrumentation                                      ---*/
 /*------------------------------------------------------------*/
 
-static IRBB* ms_instrument ( IRBB* bb_in, VexGuestLayout* layout, 
-                             IRType gWordTy, IRType hWordTy )
+static
+IRBB* ms_instrument ( VgCallbackClosure* closure,
+                      IRBB* bb_in, 
+                      VexGuestLayout* layout, 
+                      VexGuestExtents* vge,
+                      IRType gWordTy, IRType hWordTy )
 {
    /* XXX Will Massif work when gWordTy != hWordTy ? */
    return bb_in;
@@ -1247,8 +1175,6 @@ static void calc_exact_ST_dbld(ULong* heap2, ULong* heap_admin2, ULong* stack2)
 {
    UInt i, N = curr_census;
 
-   VGP_PUSHCC(VgpCalcSpacetime2);
-
    *heap2       = 0;
    *heap_admin2 = 0;
    *stack2      = 0;
@@ -1270,8 +1196,6 @@ static void calc_exact_ST_dbld(ULong* heap2, ULong* heap_admin2, ULong* stack2)
    *heap2       /= 2;
    *heap_admin2 /= 2;
    *stack2      /= 2;
-
-   VGP_POPCC(VgpCalcSpacetime2);
 }
 
 /*------------------------------------------------------------*/
@@ -1353,8 +1277,6 @@ static void write_hp_file(void)
    Char*  cmdbuf;
    Int    cmdlen;
 
-   VGP_PUSHCC(VgpPrintHp);
-   
    // Open file
    hp_file  = make_filename( base_dir, ".hp" );
    ps_file  = make_filename( base_dir, ".ps" );
@@ -1363,7 +1285,6 @@ static void write_hp_file(void)
                              VKI_S_IRUSR|VKI_S_IWUSR);
    if (sres.isError) {
       file_err( hp_file );
-      VGP_POPCC(VgpPrintHp);
       return;
    } else {
       fd = sres.val;
@@ -1371,8 +1292,13 @@ static void write_hp_file(void)
 
    // File header, including command line
    SPRINTF(buf, "JOB         \"");
-   for (i = 0; i < VG_(client_argc); i++)
-      SPRINTF(buf, "%s ", VG_(client_argv)[i]);
+   if (VG_(args_the_exename)) {
+      SPRINTF(buf, "%s", VG_(args_the_exename));
+   }
+   for (i = 0; i < VG_(args_for_client).used; i++) {
+      if (VG_(args_for_client).strs[i])
+         SPRINTF(buf, " %s", VG_(args_for_client).strs[i]);
+   }
    SPRINTF(buf, /*" (%d ms/sample)\"\n"*/ "\"\n"
                 "DATE        \"\"\n"
                 "SAMPLE_UNIT \"ms\"\n"
@@ -1441,8 +1367,6 @@ static void write_hp_file(void)
    VG_(free)(hp_file);
    VG_(free)(ps_file);
    VG_(free)(aux_file);
-
-   VGP_POPCC(VgpPrintHp);
 }
 
 /*------------------------------------------------------------*/
@@ -1561,7 +1485,7 @@ static void pp_all_XPts2(Int fd, Queue* q, ULong heap_spacetime,
 //         tl_assert(sum <= xpt->exact_ST_dbld);
 //         tl_assert(sum * 1.05 > xpt->exact_ST_dbld );
 //         if (sum != xpt->exact_ST_dbld) {
-//            VG_(printf)("%ld, %ld\n", sum, xpt->exact_ST_dbld);
+//            VG_(printf)("%lld, %lld\n", sum, xpt->exact_ST_dbld);
 //         }
       }
 
@@ -1666,8 +1590,6 @@ write_text_file(ULong total_ST, ULong heap_ST)
    Char*  text_file;
    Char*  maybe_p = ( XHTML == clo_format ? "<p>" : "" );
 
-   VGP_PUSHCC(VgpPrintXPts);
-
    // Open file
    text_file = make_filename( base_dir, 
                               ( XText == clo_format ? ".txt" : ".html" ) );
@@ -1676,7 +1598,6 @@ write_text_file(ULong total_ST, ULong heap_ST)
                              VKI_S_IRUSR|VKI_S_IWUSR);
    if (sres.isError) {
       file_err( text_file );
-      VGP_POPCC(VgpPrintXPts);
       return;
    } else {
       fd = sres.val;
@@ -1693,9 +1614,14 @@ write_text_file(ULong total_ST, ULong heap_ST)
    }
 
    // Command line
-   SPRINTF(buf, "Command: ");
-   for (i = 0; i < VG_(client_argc); i++)
-      SPRINTF(buf, "%s ", VG_(client_argv)[i]);
+   SPRINTF(buf, "Command:");
+   if (VG_(args_the_exename)) {
+      SPRINTF(buf, " %s", VG_(args_the_exename));
+   }
+   for (i = 0; i < VG_(args_for_client).used; i++) {
+      if (VG_(args_for_client).strs[i])
+         SPRINTF(buf, " %s", VG_(args_for_client).strs[i]);
+   }
    SPRINTF(buf, "\n%s\n", maybe_p);
 
    if (clo_heap)
@@ -1703,8 +1629,6 @@ write_text_file(ULong total_ST, ULong heap_ST)
 
    tl_assert(fd >= 0);
    VG_(close)(fd);
-
-   VGP_POPCC(VgpPrintXPts);
 }
 
 /*------------------------------------------------------------*/
@@ -1715,7 +1639,7 @@ static void
 print_summary(ULong total_ST, ULong heap_ST, ULong heap_admin_ST,
               ULong stack_ST)
 {
-   VG_(message)(Vg_UserMsg, "Total spacetime:   %,ld ms.B", total_ST);
+   VG_(message)(Vg_UserMsg, "Total spacetime:   %,llu ms.B", total_ST);
 
    // Heap --------------------------------------------------------------
    if (clo_heap)
@@ -1789,7 +1713,7 @@ static void ms_post_clo_init(void)
    hp_census();
 }
 
-static void ms_pre_clo_init()
+static void ms_pre_clo_init(void)
 { 
    VG_(details_name)            ("Massif");
    VG_(details_version)         (NULL);
@@ -1823,18 +1747,6 @@ static void ms_pre_clo_init()
    VG_(track_new_mem_stack_signal)( new_mem_stack_signal );
    VG_(track_die_mem_stack_signal)( die_mem_stack_signal );
 
-   // Profiling events
-   VG_(register_profile_event)(VgpGetXPt,         "get-XPt");
-   VG_(register_profile_event)(VgpGetXPtSearch,   "get-XPt-search");
-   VG_(register_profile_event)(VgpCensus,         "census");
-   VG_(register_profile_event)(VgpCensusHeap,     "census-heap");
-   VG_(register_profile_event)(VgpCensusSnapshot, "census-snapshot");
-   VG_(register_profile_event)(VgpCensusTreeSize, "census-treesize");
-   VG_(register_profile_event)(VgpUpdateXCon,     "update-XCon");
-   VG_(register_profile_event)(VgpCalcSpacetime2, "calc-exact_ST_dbld");
-   VG_(register_profile_event)(VgpPrintHp,        "print-hp");
-   VG_(register_profile_event)(VgpPrintXPts,      "print-XPts");
-
    // HP_Chunks
    malloc_list  = VG_(HT_construct)( 80021 );   // prime, big
 
@@ -1844,9 +1756,9 @@ static void ms_pre_clo_init()
    tl_assert( VG_(getcwd)(base_dir, VKI_PATH_MAX) );
 }
 
-VG_DETERMINE_INTERFACE_VERSION(ms_pre_clo_init, 0)
+VG_DETERMINE_INTERFACE_VERSION(ms_pre_clo_init)
 
 /*--------------------------------------------------------------------*/
-/*--- end                                                ms_main.c ---*/
+/*--- end                                                          ---*/
 /*--------------------------------------------------------------------*/
 
